﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Northwoods.Go;
using Northwoods.Go.Layout;
using Northwoods.Go.Xml;
using Northwoods.Go.Pdf;
using System.Resources;

using System.Drawing.Text;

namespace DemoShell {
  public partial class Form1 : Form {
    public Form1() {
      InitializeComponent();
      System.Reflection.Assembly assem = this.GetType().Assembly;
      GoImage.DefaultResourceManager = new ResourceManager("DemoShell.Properties.Resources", assem);

      goOverview1.Observed = goView1;  // be the overview for goView1
      goView1.DragsRealtime = true;

      AddSomeNodesToPalette();

      GoLink l = new GoLink();
      l.Orthogonal = true;
      l.Style = GoStrokeStyle.RoundedLineWithJumpGaps;
      goView1.NewLinkPrototype = l;

     // dofonts();
    }

    private void AddSomeNodesToPalette() {

      GoDocument pal = goPalette1.Document;

      GoGeneralNode gn = new GoGeneralNode();
      gn.Initialize(null, "start", "", "Start", 0, 1);
      gn.Icon.Size = new SizeF(40, 40);
      pal.Add(gn);

      GoGeneralNode stop = new GoGeneralNode();
      stop.Initialize(null, "stop", "", "Stop", 1, 0);
      stop.Icon.Size = new SizeF(40, 40);
      pal.Add(stop);

      GeneralNode365 n1 = new GeneralNode365();
      n1.Initialize();
      pal.Add(n1);
    }

    private GoText newText(string fontfamily, float size, bool bold, bool italic) {
      GoText t = new GoText();
      t.Editable = true;
      FontStyle fs = bold ? FontStyle.Bold : FontStyle.Regular;
      fs = italic ? FontStyle.Italic : fs;
      Font font = new Font(fontfamily, size, fs);
      t.Font = font;
      string bo = bold ? " bold" : "";
      string it = italic ? " italic" : "";
      t.Text = fontfamily +  bo + it + " " + size.ToString();
      return t;
    }

    // The object that myPropertyGrid displays the properties of
    public Object Object {
      get { return propertyGrid1.SelectedObject; }
      set {
        if (this.propertyGrid1.SelectedObject == value) return;
        if (value is GoObject)
          this.comboBoxProps.SelectedIndex = 0;
        else if (value is GoDocument)
          this.comboBoxProps.SelectedIndex = 2;
        else if (value is GoView)
          this.comboBoxProps.SelectedIndex = 3;
        this.propertyGrid1.SelectedObject = value;
      }
    }

    private void comboBoxProps_SelectedIndexChanged(object sender, EventArgs e) {
      switch (this.comboBoxProps.SelectedIndex)
      {
        default:
        case 0:
          this.Object = goView1.Selection.Primary;
          break;
        case 1:
          {
            GoObject obj = goView1.Selection.Primary;
            if (obj != null)
              this.Object = obj.Layer;
            else
              this.comboBoxProps.SelectedIndex = 0;
            break;
          }
        case 2:
          this.Object = goView1.Document;
          break;
        case 3:
          this.Object = goView1;
          break;
      }
    }
    
    // GoView Event Handlers

    private void goView1_ObjectGotSelection(object sender, GoSelectionEventArgs e) {
      Object = goView1.Selection.Primary; // set the property grid
    }

    private void toolStripButtonPDF_Click(object sender, EventArgs e) {
      GoView view = goView1;
      SaveFileDialog dlg = new SaveFileDialog();
      dlg.DefaultExt = "pdf";
      dlg.Filter = "PDF files (*.pdf)|*.pdf";
      if (dlg.ShowDialog() == DialogResult.OK) {
        GoPdfWriter writer = new GoPdfWriter();
        writer.View = view;
        writer.PageSize = GoPdfPageSizes.A_Letter;
        //writer.Margins = new GoPdfMargins(.5f);
        //writer.Scale = view.Document.ComputeBounds().Width / 7.4f;
        //writer.Alignment = GoPdfAlignment.Top | GoPdfAlignment.Left;

        GoPdfMetadata meta = new GoPdfMetadata() { Title = "GoDiagram DemoShell Diagram", Author = new List<string>() { "Author1", "Author2" }, Producer = "nwoods.com" };
        writer.Metadata = meta;
        writer.RegisterStandardGenerators();

        writer.Generate(dlg.FileName);
        MessageBox.Show("PDF Created in " + dlg.FileName);

      }
    }

    private void WriteXml(string path) {
      GoXmlWriter xw = new GoXmlWriter();
      RegisterTransformers(xw);
      xw.Objects = goView1.Document;
      using (StreamWriter file = new StreamWriter(path)) {
        xw.Generate(file);
      }
      CurrentFile = path;
    }

    private void ReadXml(string path) {
      GoXmlReader xr = new GoXmlReader();
      RegisterTransformers(xr);
      using (StreamReader file = new StreamReader(path)) {
        goView1.Document = (GoDocument)xr.Consume(file);
      }
      goView1.Document.MaintainsPartID = true;
      goView1.Document.UndoManager = new GoUndoManager();
      CurrentFile = path;
    }

    private void RegisterTransformers(GoXmlReaderWriterBase rw) {
      // create a prototype document 
      GoDocument doc = new GoDocument();
      GoXmlBindingTransformer bt = new GoXmlBindingTransformer("graph", doc);
      bt.AddBinding("lastpartid", "LastPartID");
      rw.AddTransformer(bt);

      MessageBox.Show("No GoXmlBindingTransformers for nodes or\nlinks in method RegisterTransformers", "Some work to do here");

    }

    private void printToolStripButton_Click(object sender, EventArgs e) {
      goView1.Print();
    }

    private void cutToolStripButton_Click(object sender, EventArgs e) {
      goView1.EditCut();
    }

    private void copyToolStripButton_Click(object sender, EventArgs e) {
      goView1.EditCopy();
    }

    private void pasteToolStripButton_Click(object sender, EventArgs e) {
      goView1.EditPaste();
    }

    private void newToolStripButton_Click(object sender, EventArgs e) {
      GoDocument doc = goView1.Document;
      doc.Clear();
      doc.UndoManager = new GoUndoManager();
    }

    private void openToolStripButton_Click(object sender, EventArgs e) {
      OpenFileDialog dlg = new OpenFileDialog();
      if (dlg.ShowDialog() == DialogResult.OK) {
        try {
          ReadXml(dlg.FileName);
        }
        catch (Exception ex) {
          MessageBox.Show(this, ex.Message, "Error loading graph from a file");
        }
      }
      goView1.RequestFocus();
    }

    private void saveToolStripButton_Click(object sender, EventArgs e) {
      if (CurrentFile == "")
        SaveAs();
      else {
        WriteXml(CurrentFile);
      }
    }

    private void SaveAs() {
      SaveFileDialog s = new SaveFileDialog();
      s.DefaultExt = "XML";
      s.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
      s.FileName = CurrentFile;
      if (s.ShowDialog() == DialogResult.OK) {
        try {
          WriteXml(s.FileName);
        }
        catch (Exception ex) {
          MessageBox.Show(this, ex.Message, "Error in saving file");
        }
      }
    }

    private string CurrentFile = "";

  }
}
