﻿/*
 *  Copyright © Northwoods Software Corporation, 1998-2013. All Rights
 *  Reserved.
 *
 *  Restricted Rights: Use, duplication, or disclosure by the U.S.
 *  Government is subject to restrictions as set forth in subparagraph
 *  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *  52.227-14 Alt. III, as applicable.
 */

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.Windows.Forms;
using Northwoods.Go;

namespace DemoShell
{
  [Serializable]
  public class GeneralNode365 : GoGeneralNode {
    public GeneralNode365() {
      //this.LeftPortsAlignment = Middle;
      this.LeftPortLabelsInside = true;
      this.LeftPortsLabelSpacing = 4;
      //this.RightPortsAlignment = Middle;
      this.RightPortLabelsInside = true;
      this.RightPortsLabelSpacing = 4;
    }

    // some test code...
    public void Initialize() {
      this.Resizable = false;
      //Initialize(null, @"star.gif", "top label", "bottom label", 4, 2);

      Initialize(null, @"star.gif", "top label", null, 4, 2);

      this.Image.Size = new SizeF(24, 23);
      if (this.TopLabel != null)
      {
        this.TopLabel.Editable = false;
      }
      if (this.BottomLabel != null)
      {
        this.BottomLabel.Editable = false;
      }

      GetLeftPort(0).Name = "Input1";
      GetLeftPort(1).Name = "Input2";
      GetLeftPort(2).Name = "Input3";
      GetLeftPort(3).Name = "Input4";
      GetRightPort(0).Name = "Output1";
      GetRightPort(1).Name = "Output2";

      //// create Header, not part of the Icon
      //GoGroup hdr = new GoGroup();
      //hdr.Selectable = false;
      //hdr.AutoRescales = false;
      //GoRoundedRectangle r = new GoRoundedRectangle();
      //r.Corner = new SizeF(2, 2);
      //r.Selectable = false;
      //r.Size = new SizeF(100, 20);
      //r.RoundedCornerSpots = TopLeft | TopRight;
      ////r.FillSimpleGradient(Color.Green, Color.PaleGreen, MiddleTop);
      //r.BrushColor = Color.Beige;
      //hdr.Add(r);
      //GoText abc = new GoText();
      //abc.Selectable = false;
      //abc.Text = "ABC";
      //abc.Bold = true;
      //abc.Center = r.Center;
      //hdr.Add(abc);
      //Insert(1, hdr);  // == this[1]

    }

    public GoListGroup Root
    {
      get
      {
        if (this.Count < 1) return null;
        GoListGroup list = this[0] as GoListGroup;
        return list;
      }
    }

    public GoGroup Header {
      //get { if (this.Count > 1) return this[1]; else return null; }
      get
      {
        //GoListGroup list = this.Icon as GoListGroup;
        //if (list != null && list.Count > 0)
        //{
        //  return list[0] as GoGroup;
        //}
        //return null;
        if (this.Root != null)
        {
          return this.Root.FindChild("Header") as GoGroup;
        }
        else return null;
      }
    }
    public GoRoundedRectangle Footer
    {
      get
      {
        if (this.Root != null)
        {
          return this.Root.FindChild("Footer") as GoRoundedRectangle;
        }
        else return null;
      }
    }

    public override GoObject Icon
    {
      get
      {
        // "Icon" is the second element of this[0] GoListGroup
        if (this.Root != null)
        {
          return this.Root.FindChild("Icon");   // GoGroup with GoRectangle
        }
        return null;
      }
      set
      {
        // no-op
      }
    }

    // since the GoImage isn't the Icon, but a part of the Icon,
    // we need to override this property to find the second child of the second child
    public override GoImage Image
    {
      get
      {
        GoGroup header = this.Header as GoGroup;
        if (header != null && header.Count > 1)
        {
          return header[1] as GoImage;
        }
        return null;
      }
    }

    // make sure the copied GoButton's event is set up, referring to the copied GeneralNode365's event handler
    public override GoObject CopyObject(GoCopyDictionary env) {
      GeneralNode365 info = (GeneralNode365)base.CopyObject(env);
      //GoButton but = info.Button;
      //if (but != null) {
      //  but.Action += new GoInputEventHandler(info.but_Action);
      //}
      return info;
    }

    // have the bounding selection handle go around the whole node
    public override GoObject SelectionObject {
      get { return this; }
    }

    // the Icon isn't just a GoImage, but a GoListGroup holding a two GoRectangles and a GoImage
    // in nested groups
    protected override GoObject CreateIcon(System.Resources.ResourceManager res, string iconname) {


      GoListGroup list = new GoListGroup();
      list.Selectable = false;
      list.AutoRescales = false;
      list.TopLeftMargin = new SizeF(0, 0);
      list.BottomRightMargin = new SizeF(0, 0);

      // (Header) top group with rectangle and image

      GoGroup g1 = new GoGroup();
      g1.Selectable = false;

      GoRoundedRectangle r1 = new GoRoundedRectangle();
      r1.Selectable = false;
      r1.Size = new SizeF(100, 50);
      r1.RoundedCornerSpots = TopLeft | TopRight;
      r1.Corner = new SizeF(2, 2);
      //r1.RoundedCornerSpots = NoSpot;
      //r1.FillSimpleGradient(Color.Green, Color.PaleGreen, MiddleTop);
      r1.BrushColor = Color.Pink;
      if (iconname != null)
      {
        GoImage ni = new GoImage();
        ni.Selectable = false;
        ni.AutoRescales = false;
        if (res != null)
          ni.ResourceManager = res;
        ni.Name = iconname;
        ni.Center = g1.GetSpotLocation(GoObject.Middle);

        //float imgH = this.Image != null ? this.Image.Height : 0;
        //float labH = this.TopLabel != null ? this.TopLabel.Height : 0;
        //r1.Height = (5 * 3) + imgH + labH;

        g1.Add(r1);
        g1.Add(ni);
      }
      list.Add(g1);
      list.AddChildName("Header", g1);

      // middle rectangle  (this is "Icon")
      GoGroup g2 = new GoGroup();
      g2.Selectable = false;
      GoRectangle r2 = new GoRectangle();
      r2.Selectable = false;
      r2.Size = new SizeF(100, 100);
      r2.BrushColor = Color.Beige;
      g2.Add(r2);
      list.Add(g2);
      list.AddChildName("Icon", g2);


      // "Footer" bottom rectangle for BottomLabel
      if (this.BottomLabel != null)
      {
        GoRoundedRectangle r3 = new GoRoundedRectangle();
        r3.Corner = new SizeF(2, 2);
        r3.Selectable = false;
        r3.Size = new SizeF(100, 20);
        r3.RoundedCornerSpots = BottomLeft | BottomRight;
        r3.FillSimpleGradient(Color.PaleGreen, Color.Green, MiddleTop);
        list.Add(r3);
        list.AddChildName("Footer", r3);
      }
      this.Add(list);   // make this the "Root" object for the node.
      return g2;
    }

    protected override GoGeneralNodePort CreatePort(bool input) {
      GoGeneralNodePort p = new GoGeneralNodePort();
      p.Size = new SizeF(8, 8);
      p.Style = GoPortStyle.Rectangle;
      p.BrushColor = Color.LightGray;
      p.PenColor = Color.Black;
      p.PenWidth = 1;
      p.AutoRescales = false;
      p.LeftSide = input;
      p.IsValidFrom = !input;
      p.IsValidTo = input;
      return p;
    }

    protected override GoGeneralNodePortLabel CreatePortLabel(bool input) {
      GoGeneralNodePortLabel l = new GoGeneralNodePortLabel();
      l.Editable = false;
      return l;
    }

    // center the labels in the middle of the extra rectangles in the Icon
    public override void LayoutChildren(GoObject childchanged) {
      base.LayoutChildren(childchanged);
      GoGroup icongrp = this.Icon as GoGroup;
      if (icongrp != null) {
        GoGroup header = this.Header;
        GoObject toplab = this.TopLabel;
        GoObject botlab = this.BottomLabel;
        if (icongrp.Count > 0) {
          int num = this.LeftPortsCount;
          float portandlabelheight = 0; // total height
          for (int i = 0; i < num; i++) {
            GoGeneralNodePort p = GetLeftPort(i);
            if (p != null && p.Visible)
              portandlabelheight += p.PortAndLabelHeight;
          }
          float h = portandlabelheight;

          // now consider the ports on the right side
          num = this.RightPortsCount;
          portandlabelheight = 0; // total height
          for (int i = 0; i < num; i++) {
            GoGeneralNodePort p = GetRightPort(i);
            if (p != null && p.Visible)
              portandlabelheight += p.PortAndLabelHeight;
          }
          h = Math.Max(h, portandlabelheight);
          h += 10;  // spacing
          icongrp[0].Height = h;
        }
        //if (header != null && icongrp.Count > 0) {
        //  header.SetSpotLocation(MiddleBottom, icongrp, MiddleTop);
        //}
        this.LayoutLeftPorts();
        this.LayoutRightPorts();

        GoImage img = this.Image;
        if (header != null && toplab != null && img != null)
        {
          GoRoundedRectangle rr = header[0] as GoRoundedRectangle;
          if (rr != null)
          {
              //img.Center = new PointF(rr.Location.Y + 5 + this.Image.Height / 2, icongrp[0].Center.X);
              //toplab.Center = new PointF(rr.Bottom - 5 + toplab.Height / 2, icongrp[0].Center.X);
            img.SetSpotLocation(MiddleTop, rr, MiddleTop, new SizeF(0, 5));
            toplab.SetSpotLocation(MiddleBottom, rr, MiddleBottom, new SizeF(0, -5));
          }
          
        }
        if (botlab != null && icongrp.Count > 2) {
          botlab.Center = icongrp[2].Center;
        }
      }
    }
  }
}
